# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2009- Spyder Project Contributors
#
# Distributed under the terms of the MIT License
# (see spyder/__init__.py for details)
# -----------------------------------------------------------------------------


"""
spyder.widgets
==============

Widgets defined in this module may be used in any other Qt-based application.

They are also used in Spyder through the Plugin interface (see spyder.plugins).
"""
